#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright (C) 2015 Sergey Poznyakoff
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from wikimarkup import *
from wikins import wiki_ns_re, wiki_ns
import re
import urllib

class TexiWikiMarkup (WikiMarkup):
    sectcomm = {
        'numbered': [
            '@top',
            '@chapter',      
            '@section',      
            '@subsection',   
            '@subsubsection'
        ],
        'unnumbered': [
            '@top',
            '@unnumbered',
            '@unnumberedsec',
            '@unnumberedsubsec',
            '@unnumberedsubsubsec'
        ],
        'appendix': [
            '@top',
            '@appendix',
            '@appendixsec',
            '@appendixsubsec',
            '@appendixsubsubsec'
        ],
        'heading': [
            '@majorheading'
            '@chapheading',
            '@heading',
            '@subheading',
            '@subsubheading'
        ]
    }

    sectioning_model = 'numbered'
    sectioning_start = 0

    def __init__(self, *args, **keywords):
        super(TexiWikiMarkup, self).__init__(*args, **keywords)
        if "sectioning-model" in keywords:
            val = keywords["sectioning-model"]
            if val in self.sectcomm:
                self.sectioning_model = val
            else:
                raise ValueError("Invalid value for sectioning model: %s" % val)
        if "sectioning-start" in keywords:
            val = keywords["sectioning-start"]
            if val < 0 or val > 4:
                raise ValueError("Invalid value for sectioning start: %s" % val)
            else:
                self.sectioning_start = val

    replchars = re.compile(r'([@{}])')
    acc = None

    def _print(self, text, **kw):
        nl = kw.pop('nl', False)
        escape = kw.pop('escape', True)
        if nl and not self.acc.endswith('\n'):
            self.acc += '\n'
        if text:
            if escape:
                self.acc += self.replchars.sub(r'@\1', text)
            else:
                self.acc += text

    def _begin_print(self, val = ''):
        s = self.acc
        self.acc = val
        return s

    def _end_print(self, val = None):
        s = self.acc
        self.acc = val
        return s
    
    def __str__(self):
        self._begin_print()
        for elt in self.tree:
            self.format(elt)
        return self._end_print()

    def format(self, elt):
        if elt['type'] == 'TEXT':
            if isinstance(elt['content'],list):
                for s in elt['content']:
                    self._print(s)
            else:
                self._print(elt['content'])
        elif elt['type'] == 'TAG':
            self.str_tag(elt)
        elif elt['type'] == 'PARA':
            self.str_para(elt)
        elif elt['type'] == 'PRE':
            self.str_pre(elt)
        elif elt['type'] == 'IT':
            self.str_it(elt)
        elif elt['type'] == 'BOLD':
            self.str_bold(elt)
        elif elt['type'] == 'LINK':
            self.str_link(elt)
        elif elt['type'] == 'TMPL':
            self.str_tmpl(elt)
        elif elt['type'] == 'BAR':
            self.str_bar()
        elif elt['type'] == 'HDR':
            self.str_hdr(elt)
        elif elt['type'] == 'REF':
            self.str_ref(elt)
        elif elt['type'] == 'ENV':
            self.str_env(elt)
        elif elt['type'] == 'IND':
            self.str_ind(elt)
        elif elt['type'] == 'SEQ':
            for x in elt['content']:
                self.format(x)
        else:
            self._print(str(elt))

    def str_tag(self, elt):
        if elt['tag'] in ['code', 'tt']:
            save = self._begin_print()
            self.nested += 1
            self.format(elt['content'])
            self.nested -= 1
            s = self._end_print(save)
            if elt['isblock']:
                self._print('@example', nl=True, escape=False)
                self._print(s, escape=False)
                self._print('@end example\n', nl=True, escape=False)
            else:
                self._print('@code{%s}' % s, escape=False)
        elif elt['tag'] == 'div':
            if 'args' in elt and 'id' in elt['args']:
                self._print("@anchor{%s}\n" % elt['args']['id'],
                            nl=True, escape=False)
            self.format(elt['content'])
        else:
            self._print('<' + elt['tag'])
            if elt['args']:
                self._print(' ' + elt['args'])
            self._print('>');
            self.format(elt['content']);
            self._print('</' + elt['tag'] + '>')
 
    def str_para(self, elt):
        if self.acc and not self.acc.endswith('\n\n'):
            self._print('\n', nl=True)
        for x in elt['content']:
            self.format(x)
        if self.acc and not self.acc.endswith('\n\n'):
            self._print('\n', nl=True)
        
    def str_pre(self, elt):
        if not self.nested:
            self._print('@example\n', nl=True, escape=False)
        for x in elt['content']:
            self.format(x)
        if not self.nested:
            self._print('@end example\n', nl=True, escape=False)

    def concat(self, eltlist):
        for x in eltlist:
            self.format(x)
    
    def str_it(self, elt):
        self._print('@i{', escape=False)
        self.concat(elt['content'])
        self._print('}', escape=False)

    def str_bold(self, elt):
        self._print('@b{', escape=False)
        self.concat(elt['content'])
        self._print('}', escape=False)

    def str_hdr(self, elt):
        level = elt['level']
        if level > len(self.sectcomm[self.sectioning_model]) - 1 - self.sectioning_start:
            self._print("@* ", nl=True, escape=False)
            self.format(elt['content'])
        else:
            self._print(self.sectcomm[self.sectioning_model][level - self.sectioning_start] + " ", nl=True, escape=False)
            self.format(elt['content'])
            self._print(None, nl=True)
            if self.sectcomm[self.sectioning_model][0] == '@top':
                self._print('@node ', nl=True, escape=False)
                self.format(elt['content'])
                self._print('\n')
        self._print(None, nl=True)
        
    def str_bar(self):
        self._print("\n-----\n")

    def str_ind(self, elt):
        self._print("@w{ }" * elt['level'], nl=True, escape=False)
        self.format(elt['content'])
        self._print(None, nl=True)

    def str_env(self, elt):
        if elt['envtype'] == 'unnumbered':
            self._print('@itemize @bullet\n', nl=True, escape=False)
            for s in elt['content']:
                self._print('@item ', nl=True, escape=False)
                self.format(s['content'])
                self._print(None, nl=True)
                self._print('\n')
            self._print('@end itemize\n', nl=True, escape=False)
        elif elt['envtype'] == 'numbered':
            self._print('@enumerate\n', nl=True, escape=False)
            for s in elt['content']:
                self._print('@item ', nl=True, escape=False)
                self.format(s['content'])
                self._print(None, nl=True)
                self._print('\n')
            self._print('@end enumerate\n', nl=True, escape=False)
        elif elt['envtype'] == 'defn':
            self._print('@table @asis\n', nl=True, escape=False)
            for s in elt['content']:
                if s['subtype'] == 0:
                    self._print('@item ', nl=True, escape=False)
                    self.format(s['content'])
                    self._print(None, nl=True)
                else:
                    self.format(s['content'])
                    self._print(None, nl=True)
                    self._print('\n')
            self._print('@end table\n', nl=True, escape=False)

    def str_link(self, elt):
        save = self._begin_print()
        self.format(elt['content'][0])
        arg = self._end_print()
        if len(elt['content']) > 1:
            s = []
            for x in elt['content'][0:2]:
                self._begin_print()
                self.format(x)
                s.append(self._end_print())
            text = s[1]
        else:
            s = None
            text = None

        self._end_print(save)

        if s:
            if s[0] == 'disambigR' or s[0] == 'wikiquote':
                return
            if len(s) > 1 and s[1] == 'thumb':
                return

        (qual,sep,tgt) = arg.partition(':')
        if text:
            self._print("@ref{%s,%s}" % (qual, text), escape=False)
        else:
            self._print("@ref{%s}" % qual, escape=False)
    
    def str_tmpl(self, elt):
        self._print("FIXME: str_tmpl not implemented\n")
            
    def str_ref(self, elt):
        target = elt['ref']
        save = self._begin_print()
        self.format(elt['content'])
        text = self._end_print(save)
        if text and text != '':
            self._print("@uref{%s,%s}" % (target, text), escape=False)
        else:
            self._print("@uref{%s}" % target, escape=False)
    
    
        
    
    
